 
const char *GridFrag = 
"#version 330 core\n"
"//\n"
"// Near and far gives 2 points on the camera ray for a particular fragment, we can then compute a so ray intersection with the\n"
"// plane in the fragment shader. Using this ray, [pos = near + alpha*(far-near)] to compute the intersection with the ground plane\n"
"// (pos.y=0 or pos.z=0) The position of the plane on the ray is then alpha = -near.y/(far.y-near.y) alpha < 0 means it intersects\n"
"// behind the camera, so we won't see it in front, so we can discard the fragment\n"
"//\n"
"out vec4 FragColor;\n"
"in vec3 near;\n"
"in vec3 far;\n"
"in mat4 model;\n"
"in mat4 proj;\n"
"uniform float planeOrientation;\n"
"void main()\n"
"{\n"
"    // cellSize could be a function parameter, to display grids at multiple levels\n"
"    const float cellSize = 1000.0;\n"
"    float distanceToPlane = mix(-near.z/(far.z-near.z), -near.y/(far.y-near.y), planeOrientation);\n"
"    if (distanceToPlane < 0) discard;\n"
"    // plane_pt is the actual point on the plan\n"
"    vec3 pointOnPlane = near + distanceToPlane * (far - near);\n"
"    vec2 gridCoord = mix(pointOnPlane.xy, pointOnPlane.xz, planeOrientation);\n"
"    // We need the sum of the absolute value of derivatives in x and y to get the same projected line widt\n"
"    vec2 gridCoordDerivative = fwidth(gridCoord);\n"
"    float fadeFactor = length(gridCoordDerivative)/(cellSize/2.f);\n"
"    vec2 gridLine = 1.f - abs(clamp(mod(gridCoord, cellSize) / (2.0*gridCoordDerivative), 0.0, 1.0) * 2 - 1.f);\n"
"    float gridLineAlpha = max(gridLine.x, gridLine.y);\n"
"    vec4 c = vec4(0.7, 0.7, 0.7, 1.0);\n"
"    c.a = max(0, gridLineAlpha * (1-fadeFactor*fadeFactor));\n"
"    if (c.a == 0) discard;\n"
"    FragColor = c;\n"
"    // planeProj is the projection of the plane point on this fragment, it allows to compute its depth\n"
"    vec4 planeProj = proj*model*vec4(pointOnPlane, 1.0);\n"
"    gl_FragDepth = (((gl_DepthRange.far-gl_DepthRange.near) * (planeProj.z/planeProj.w)) + gl_DepthRange.near + gl_DepthRange.far) / 2.0;\n"
"}\n"
;
